@@ -48,6 +48,11 @@ SOFTWARE.
 //!  * `tokio` @ 1.6.0 (but should also work with 1.0.0)
 //!  * `rustc` @ 1.52.1 (but should also work with 1.45.2)
 
+use std::collections::HashSet;
+use std::sync::atomic::AtomicU64;
+use std::sync::Mutex;
+
+use lazy_static::lazy_static;
 use tokio::time::Duration;
 
 /// **INTERNAL** Use macro [`set_timeout`] instead!
@@ -59,11 +64,32 @@ pub async fn _set_timeout(f: impl Fn(), ms: u64) {
     f();
 }
 
-/// **INTERNAL** Use macro [`set_interval`] instead!
-///
+/// **INTERNAL** Used to manage intervals created by macro [`set_interval`]!
+/// Helps to assign unique IDs to intervals and stop them.
+pub struct IntervalManager {
+    /// Monotonic incrementing counter from 0 to max value used for interval IDs.
+    pub counter: AtomicU64,
+    /// Contains only the IDs of dispatched and not cleared intervals.
+    pub running_intervals: Mutex<HashSet<u64>>,
+}
+
+lazy_static! {
+    /// **INTERNAL** Used to manage intervals created by macro [`set_interval`]!
+    pub static ref INTERVAL_MANAGER: IntervalManager = IntervalManager {
+        counter: AtomicU64::new(0),
+        running_intervals: Mutex::new(HashSet::new())
+    };
+}
+
 /// Creates a future that glues the tokio interval function together with
-/// the provided callback.
-pub async fn _set_interval(f: impl Fn(), ms: u64) {
+/// the provided callback. Helper function for [`_set_interval_spawn`].
+async fn _set_interval(f: impl Fn() + Send + 'static, ms: u64, id: u64) {
+    // own scope -> early drop lock
+    {
+        let mut map = INTERVAL_MANAGER.running_intervals.lock().unwrap();
+        map.insert(id);
+    }
+
     let mut int = tokio::time::interval(Duration::from_millis(ms));
 
     // the first tick in tokios interval returns immediately. Because we want behaviour
@@ -75,10 +101,38 @@ pub async fn _set_interval(f: impl Fn(), ms: u64) {
     // when the tokio runtime gets dropped.
     loop {
         int.tick().await;
+
+        // breaks the loop
+        let map = INTERVAL_MANAGER.running_intervals.lock().unwrap();
+        if !map.contains(&id) {
+            break;
+        }
+
         f();
     }
 }
 
+/// **INTERNAL** Use macro [`set_interval`] instead!
+///
+/// Acquires a new ID, creates an interval future and returns the ID. The future gets
+/// dispatched as tokio task.
+pub fn _set_interval_spawn(f: impl Fn() + Send + 'static, ms: u64) -> u64 {
+    let id = INTERVAL_MANAGER
+        .counter
+        .fetch_add(1, std::sync::atomic::Ordering::SeqCst);
+    tokio::spawn(_set_interval(f, ms, id));
+    id
+}
+
+/// Clears an interval that was created via [`set_interval!`]. You have to pass
+/// the unique numeric ID as parameter. Throws no error, if the ID is unknown.
+pub fn clear_interval(id: u64) {
+    let mut set = INTERVAL_MANAGER.running_intervals.lock().unwrap();
+    if set.contains(&id) {
+        set.remove(&id);
+    }
+}
+
 /// Creates a timeout that behaves similar to `setTimeout(callback, ms)` in Javascript
 /// for the `tokio` runtime. Unlike in Javascript, it will only be executed, if after the
 /// specified time passed, the `tokio` runtime still lives, i.e. didn't got dropped.
@@ -124,11 +178,11 @@ macro_rules! set_timeout {
     };
     // match for expr, like `set_timeout!(println!())`
     ($cb:expr, $ms:literal) => {
-        tokio::spawn($crate::_set_timeout(|| $cb, $ms));
+        $crate::set_timeout!(|| $cb, $ms);
     };
     // match for block
     ($cb:block, $ms:literal) => {
-        tokio::spawn($crate::_set_timeout(|| $cb, $ms));
+        $crate::set_timeout!(|| $cb, $ms);
     };
 }
 
@@ -140,10 +194,8 @@ macro_rules! set_timeout {
 /// You don't get a handle to manually wait for it, you must ensure, that the tokio
 /// runtime lives long enough.
 ///
-/// The interval is not stoppable (unless the application stops). Strictly speaking, there
-/// should be a way to clear an interval, similar to javascript. But because this library is
-/// mainly for educational reasons and for "low priority indefinetely running background tasks",
-/// this is not implemented yet. If you need it, create an issue or a PR.
+/// The macro returns an numeric ID. Similar to Javascript, you can use thie ID to
+/// clear/stop intervals with [`clear_interval`].
 ///
 /// # Parameters
 /// * `#1` expression, closure-expression, block or identifier (which points to a closure).
@@ -158,6 +210,8 @@ macro_rules! set_timeout {
 /// #[tokio::main]
 /// async fn main() {
 ///     set_interval!(println!("hello1"), 50);
+///     // If you want to clear the interval later: save the ID
+///     // let id = set_interval!(println!("hello1"), 50);
 ///     println!("hello2");
 ///     // prevent that tokios runtime gets dropped too early
 ///     // "hello1" should get printed 2 times (50*2 == 100 < 120)
@@ -168,37 +222,33 @@ macro_rules! set_timeout {
 macro_rules! set_interval {
     // match for identifier, i.e. a closure, that is behind a variable
     ($cb:ident, $ms:literal) => {
-        tokio::spawn($crate::_set_interval($cb, $ms));
+        // not so nice, need to wrap the identifier in another closure
+        $crate::set_interval!(move || $cb(), $ms)
     };
     // match for direct closure expression
     (|| $cb:expr, $ms:literal) => {
-        tokio::spawn($crate::_set_interval(|| $cb, $ms));
+        $crate::_set_interval_spawn(|| $cb, $ms)
     };
     // match for direct move closure expression
     (move || $cb:expr, $ms:literal) => {
-        tokio::spawn($crate::_set_interval(move || $cb, $ms));
+        $crate::_set_interval_spawn(move || $cb, $ms)
     };
     // match for expr, like `set_interval!(println!())`
     ($cb:expr, $ms:literal) => {
-        tokio::spawn($crate::_set_interval(|| $cb, $ms));
+        $crate::set_interval!(|| $cb, $ms);
     };
     // match for block
     ($cb:block, $ms:literal) => {
-        tokio::spawn($crate::_set_interval(|| $cb, $ms));
+        $crate::set_interval!(|| $cb, $ms);
     };
 }
 
 #[cfg(test)]
 mod tests {
-
-    use super::*;
     use std::sync::atomic::{AtomicU64, Ordering};
     use std::sync::Arc;
 
-    #[test]
-    fn it_works() {
-        assert_eq!(2 + 2, 4);
-    }
+    use super::*;
 
     #[tokio::test]
     async fn test_set_timeout_macro_all_argument_variants_builds() {
@@ -248,7 +298,10 @@ mod tests {
         println!("hello3");
         set_timeout!(println!("hello4"), 0);
         println!("hello5");
-        // give the timeout enough execution time
+
+        // give enough execution time, before tokio gets dropped
+        // two tokio sleeps: timeouts will usually get scheduled in between
+        tokio::time::sleep(Duration::from_millis(1)).await;
         tokio::time::sleep(Duration::from_millis(1)).await;
     }
 
@@ -257,9 +310,11 @@ mod tests {
     /// be executed manually.
     #[tokio::test]
     async fn test_set_interval_visual() {
-        set_interval!(println!("should be printed 3 times"), 2);
+        let id = set_interval!(println!("should be printed 3 times"), 50);
         // give the timeout enough execution time
-        tokio::time::sleep(Duration::from_millis(7)).await;
+        tokio::time::sleep(Duration::from_millis(151)).await;
+        tokio::time::sleep(Duration::from_millis(0)).await;
+        println!("interval id is: {}", id);
     }
 
     #[tokio::test]
@@ -309,4 +364,27 @@ mod tests {
         tokio::time::sleep(Duration::from_millis(180)).await;
         assert_eq!(counter.load(Ordering::SeqCst), 3);
     }
+
+    #[tokio::test]
+    async fn test_clear_interval() {
+        let counter = Arc::new(AtomicU64::new(0));
+        let interval_id;
+        {
+            let counter = counter.clone();
+            // the block is required to change the return type to "()"
+            interval_id = set_interval!(
+                move || {
+                    counter.fetch_add(1, Ordering::SeqCst);
+                },
+                50
+            );
+        }
+
+        // give the tokio runtime enough execution time to execute the interval 3 times
+        tokio::time::sleep(Duration::from_millis(180)).await;
+        assert_eq!(counter.load(Ordering::SeqCst), 3);
+        clear_interval(interval_id);
+        tokio::time::sleep(Duration::from_millis(200)).await;
+        assert_eq!(counter.load(Ordering::SeqCst), 3);
+    }
 }
